@@ -21,7 +21,6 @@ import re
 import tempfile
 import shutil
 import copy
-import distutils.version as _V
 import datetime
 import csv
 import tarfile
@@ -33,6 +32,7 @@ import yaml
 import __version__
 import spec
 import logger
+from vercmp import FairVersion as _V
 
 SERIES_PATH = 'series.conf'
 
@@ -53,6 +53,7 @@ BOOLNO_KEYS = ('Check',
                'SupportOtherDistros',
                'NoAutoReq',
                'NoAutoProv',
+               'NoAutoReqProv',
                'NoSetup',
                'NoAutoLocale',
                'AsWholeName',
@@ -85,6 +86,7 @@ LIST_KEYS = ('Sources',
              'AutoSubPackages',
              'Files',
              'Documents',
+             'RunFdupes',
              'RpmLintIgnore'
              )
 
@@ -109,7 +111,6 @@ STR_KEYS =  ('Name',
              'LocaleOptions',
              'FilesInput',
              'PostScripts',
-             'RunFdupes',
              'Prefix',
             )
 
@@ -138,6 +139,7 @@ SUBAVAIL_KEYS = ('Name',
                  'Obsoletes',
                  'NoAutoReq',
                  'NoAutoProv',
+                 'NoAutoReqProv',
                  'FilesInput',
                  'Version', 'Release', 'Epoch', 'URL', 'BuildArch' # very rare
                 )
@@ -189,7 +191,7 @@ class GitAccess():
         return tags
 
     def get_toptag(self):
-        vers = [_V.LooseVersion(tag) for tag in self._gettags()]
+        vers = [_V(tag) for tag in self._gettags()]
         if vers:
             vers.sort()
             return str(vers[-1])
@@ -247,7 +249,9 @@ class RPMWriter():
             logger.error('Cannot read file: %s' % yaml_fpath)
 
     def dump(self):
-        print yaml.dump(yaml.load(self.stream))
+        # debugging
+        import pprint
+        pprint.pprint(yaml.dump(yaml.load(self.stream)))
 
     def _check_dup_files(self, files):
         # try to remove duplicate '%defattr' in files list
@@ -425,6 +429,13 @@ class RPMWriter():
                 if warn:
                     logger.warning('Group \'%s\' is not in the list of approved groups. See /usr/share/spectacle/GROUPS for the complete list.' % (metadata['Group']))
 
+        def _check_key_license(metadata):
+            # warning for gplv3
+            gpl3_re = re.compile('L?GPL\s*v3', re.I)
+            if metadata.has_key("License"):
+                if gpl3_re.search(metadata['License']):
+                    logger.warning('GPLv3 related license might be unacceptable.')
+
         def _check_key_epoch(metadata):
             if 'Epoch' in metadata:
                 logger.warning('Please consider to remove "Epoch"')
@@ -594,19 +605,25 @@ class RPMWriter():
             if "SubPackages" in self.metadata:
                 for sp in self.metadata["SubPackages"]:
                     if not _check_listkey(sp, key):
-                        logger.warning('the value of "%s" in %s sub-package is expected as list typed' % (key, sp['Name']))
+                        logger.warning('the value of "%s" in "%s" sub-package is expected as list typed' % (key, sp['Name']))
                         sp[key] = [sp[key]]
 
         # checking for STR expected keys
         for key in STR_KEYS:
             if not _check_strkey(self.metadata, key):
                 logger.warning('the value of "%s" in main package is expected as string typed' % key)
-                self.metadata[key] = ' '.join(self.metadata[key])
+                if isinstance(self.metadata[key], list):
+                    self.metadata[key] = ' '.join(self.metadata[key])
+                else:
+                    del self.metadata[key]
             if "SubPackages" in self.metadata:
                 for sp in self.metadata["SubPackages"]:
                     if not _check_strkey(sp, key):
-                        logger.warning('the value of "%s" in %s sub-package is expected as string typed' % (key, sp['Name']))
-                        sp[key] = ' '.join(sp[key])
+                        logger.warning('the value of "%s" in "%s" sub-package is expected as string typed' % (key, sp['Name']))
+                        if isinstance(sp[key], list):
+                            sp[key] = ' '.join(sp[key])
+                        else:
+                            del sp[key]
 
         # checking for BOOL expected keys
         for key in BOOL_KEYS:
@@ -617,7 +634,7 @@ class RPMWriter():
             if "SubPackages" in self.metadata:
                 for sp in self.metadata["SubPackages"]:
                     if not _check_boolkey(sp, key):
-                        logger.warning('the value of "%s" in %s sub-package is expected as bool typed, dropped!' % (key, sp['Name']))
+                        logger.warning('the value of "%s" in "%s" sub-package is expected as bool typed, dropped!' % (key, sp['Name']))
                         del sp[key]
 
         ######### checkings for special keys ##########
@@ -654,13 +671,12 @@ class RPMWriter():
             if len(pcbr) > 0:
                 if self.metadata.has_key('PkgConfigBR'):
                     pcbr.extend(self.metadata['PkgConfigBR'])
-                print """
-Proposal (multiple values skipped, please insert them manually):
+                logger.info("""Proposal (multiple values skipped, please insert them manually):
 PkgConfigBR:
     - %s
 PkgBR:
     - %s
-                    """ %('\n    - '.join(pcbr), '\n    - '.join(br))
+                    """ %('\n    - '.join(pcbr), '\n    - '.join(br)))
 
         _check_key_epoch(self.metadata)
 
@@ -670,6 +686,12 @@ PkgBR:
             for sp in self.metadata["SubPackages"]:
                 _check_key_group(sp)
 
+        # checking for meego invalid licenses
+        _check_key_license(self.metadata)
+        if "SubPackages" in self.metadata:
+            for sp in self.metadata["SubPackages"]:
+                _check_key_license(sp)
+
         # checking for validation of 'Description'
         if not _check_key_desc(self.metadata):
             logger.warning('main package has no qualified "Description" tag')
@@ -804,7 +826,6 @@ PkgBR:
                 if len(xx) > 1 and xx[1] == "in":
                     extractfile
                     buf = tarinfo.tobuf()
-                    #print buf
 
         tarball = None
         for uri in self.metadata['Sources']:
@@ -1174,11 +1195,11 @@ PkgBR:
             i = i.strip()
             if line_num < 3:
                 if line_num == 2:
-                    m = re.match("^#.*spectacle version ([\d.]+)(~pre|git)?$", i)
+                    m = re.match("^#.*spectacle version (\S+)$", i)
                     if m:
                         version = m.group(1)
-                        spec_ver = _V.LooseVersion(version)
-                        cur_ver = _V.LooseVersion(__version__.VERSION)
+                        spec_ver = _V(version)
+                        cur_ver = _V(__version__.VERSION)
                         if cur_ver < spec_ver:
                             logger.warning('!!! Current spectacle version is lower than the one used for this package last time')
                             answer = logger.ask('Please upgrade your spectacle firstly, continue?', False)
